# Docker AtomicService

In the following, we will package **each** Xeditor component in **one** Docker image. This has some advantages and disadvantages. We recommend using this for a productive configuration, as the main advantage is simplicity for high availability and high performance. However, to host this in a truly productive way, we recommend using Kubernetes instead of "docker-compose". We will not discuss Kubernetes further here.

In order to handle the requests correctly we are using the reverse proxy nginx. Nginx will pass all calls to `<url>/api/**` to the middleware server and trys to load static files for `<url>/**`. For this reason we have to adjust in the editor that all requests should go to `<url>/api`. There change the following line (Marked with `-`) like following (Marked with `+`) in `/src/js/index.js`:

```js
- var provider = Ext.ux.xeditor.provider.AjaxProvider.forMiddleware(BLACKBOX_URL);
+ var provider = Ext.ux.xeditor.provider.AjaxProvider.forMiddleware(BUILD_MODE === "develop" ? BLACKBOX_URL: (location.origin + "/api"));
``` 

In addition, we need the possibility to determine the build mode used and also to be able to configure the license key later. For this add the `BUILD_MODE` and `LICENSE` env variable (Marked with `+`) to your `/xepack.config.js`:

```js
// define environment variables
const env = {
+	'BUILD_MODE': process.env.XEDITOR_BUILD_MODE || "develop",
+	'LICENSE': process.env.XEDITOR_LICENSE || "",
  // ...
};
``` 

Once this is done, you can add the folder `env` from this project to your project at top level. This folder now contains almost everything needed to build all Docker images. You will of course need a current version of `docker` and `docker-compose` on your system. 

The folder `env/middleware` contains a `package.json` to defines the version of the middleware for productive use and a `Dockerfile` to build the image for the middleware. The `env/mock-server` contains a small backend server which can be used for test/demo purposes. For a productive use this is not needed. The nginx configuration can be found in `env/nginx`. This will build a very simple nginx image to work as reverse proxy. Last but not least you will find the configuration for xeditor in `env/xeditor`. 


You can build the image itself using the following commands (Make sure to replace the `--build-arg` values with your arguments). Each command is building one image: 

```sh
  # build the xeditor itself
  docker build -t xeditor -f ./env/xeditor/Dockerfile \
    --build-arg NPM_USER=YOUR_NPM_USERNAME \
    --build-arg NPM_PASSWORD=YOUR_NPM_PASSWORD \
    --build-arg XEDITOR_LICENSE_KEY=YOUR_LICENSE_KEY \
    .
  
  # Build your middleware server
  docker build -t middleware -f ./env/middleware/Dockerfile \
    --build-arg NPM_USER=YOUR_NPM_USERNAME \
    --build-arg NPM_PASSWORD=YOUR_NPM_PASSWORD \
    .

  docker build -t nginx -f ./env/nginx/Dockerfile . # Optional - You can also add this part to your reverse proxy
```

Afterwards, this containers can be published. When publishing, please always make sure **not to publish it publicly**, but always privately (DockerHub requires a paid account for this).

To run the Xeditor later I added a `docker-compose.yml` here. Using this file you can now also start the editor locally by executing the command: `docker-compose -f ./env/docker-compose.yml up`.

If this is to be run on a server later, you can simply publish the image (or export/import as `.tar.gz`) and then comment out the `build` part and use the `image` part. Also, you can run this in the background using the `-d` argument.